# Copyright (C) 2006-2008 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from ClusterModel.Device import Device
from conga_constants import LUCI_DEBUG_MODE
from LuciSyslog import get_logger

FD_VAL_FAIL = 1
FD_VAL_SUCCESS = 0

FD_NEW_SUCCESS = 'New %s successfully added to cluster'
FD_UPDATE_SUCCESS = 'Fence device %s successfully updated'
FD_NEW_FAIL = 'No agent type %s in shared device list'

FI_NEW_FAIL = 'No instance type %s in fence instance list'

FD_PROVIDE_AGENT = 'A valid agent type must be provided for each Fence Device'
FD_PROVIDE_NAME = 'A unique name must be provided for each Fence Device'
FD_PROVIDE_PATH = 'An xCAT path must be provided for each xCAT Fence Device'
FD_PROVIDE_SERVER = 'A server address must be provided for this Fence Device'
FD_PROVIDE_CSERVER = 'A cserver address must be provided for this Egenera Fence Device'
FD_PROVIDE_IP = 'An IP address must be provided for this Fence Device'
FD_PROVIDE_HOSTNAME = 'A host name must be provided for this Fence Device'
FD_PROVIDE_LOGIN = 'A login name must be provided for this Fence Device'
FD_PROVIDE_PASSWD = 'A password must be provided for this Fence Device'
FD_PROVIDE_DEVICE = 'A device must be provided for this Fence Device'
FD_PROVIDE_PORT = 'A port must be provided for this Fence Device'
FD_PROVIDE_IPMILAN_AUTH = 'Authentication type must be "none" (or blank), "md5", or "password"'
FD_PROVIDE_VMLOGIN = 'A VMware ESX management login name must be provided for this Fence Device'
FD_PROVIDE_VMPASSWD = 'A VMware ESX management password must be provided for this Fence Device'

FI_PROVIDE_PARENT = 'A Fence Device name must be provided for this Fence'
FI_PROVIDE_SWITCH = 'A switch address must be provided for this Fence'
FI_PROVIDE_PORT = 'A port value must be provided for this Fence'
FI_PROVIDE_BLADE = 'A Blade must be specified for this Fence'
FI_PROVIDE_DOMAIN = 'A Domain must be specified for this Fence'
FI_PROVIDE_IPADDRESS = 'An IP address must be provided for this Fence'
FI_PROVIDE_ELPAN = 'A LPAN value must be provided for this Egenera Fence'
FI_PROVIDE_EPSERVER = 'A PServer value must be provided for this Egenera Fence'
FI_PROVIDE_NODENAME = 'A Node Name value must be provided for this SCSI Fence'
FI_PROVIDE_PARTITION = 'A partition name must be provided for this LPAR Fence'
FI_PROVIDE_MANAGED = 'A managed system name must be provided for this LPAR Fence'

ILLEGAL_CHARS_REPLACED = 'Illegal characters were replaced by underscores. Feel free to set a new value.'

luci_log = get_logger()

def makeNCName(name):
	### name must conform to relaxNG ID type ##
	import re
	ILLEGAL_CHARS = re.compile(':| ')
	return ILLEGAL_CHARS.sub('_', name)

def validateNewFenceDevice(form, model):
	from ClusterModel.FenceDevice import FenceDevice
	fencedev = FenceDevice()

	try:
		ret = validate_fencedevice(form, model, fencedev)
		if len(ret) < 1:
			model.addFenceDevice(fencedev)
			model.setModified(True)
			return (FD_VAL_SUCCESS, fencedev.getAttribute('name'))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VNFD0: %r %s' % (e, str(e)))
		ret = [ FD_PROVIDE_AGENT ]

	return (FD_VAL_FAIL, ret)

def validateFenceDevice(form, model):
	try:
		old_fence_name = form['orig_name'].strip()
		if not old_fence_name:
			raise Exception, 'blank'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VFD0: %r %s' % (e, str(e)))
		return (FD_VAL_FAIL, [ FD_PROVIDE_NAME ])

	fencedev = None
	try:
		fencedev = model.getFenceDeviceByName(old_fence_name)
		if fencedev is None:
			raise Exception, 'fencedev is None'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VFD1: %r %s' % (e, str(e)))
		return (FD_VAL_FAIL, [ FD_PROVIDE_NAME ])

	try:
		ret = validate_fencedevice(form, model, fencedev, fence_edit=True)
		if len(ret) < 1:
			model.setModified(True)
			return (FD_VAL_SUCCESS, fencedev.getAttribute('name'))
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('VFD2: %r %s' % (e, str(e)))
		ret = [ FD_PROVIDE_NAME ]

	return (FD_VAL_FAIL, ret)

def val_apc_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_wti_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_brocade_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_vixel_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_mcdata_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_gnbd_fd(form, fencedev):
	errors = list()

	try:
		server = form['servers'].strip()
		if not server:
			raise Exception, 'blank'
		fencedev.addAttribute('servers', server)
	except Exception, e:
		errors.append(FD_PROVIDE_SERVER)

	return errors

def val_egenera_fd(form, fencedev):
	errors = list()

	try:
		cserver = form['cserver'].strip()
		if not cserver:
			raise Exception, 'blank'
		fencedev.addAttribute('cserver', cserver)
	except Exception, e:
		errors.append(FD_PROVIDE_CSERVER)

	return errors

def val_sanbox2_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_bladecenter_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_bullpap_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_lpar_fd(form, fencedev):
	errors = list()

	# fence_lpar currently requires the use of SSH
	fencedev.addAttribute('secure', '1')

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)
	return errors

def val_vmware_soap_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	try:
		ipport = form['ipport'].strip()
		if ipport:
			ipport = int(ipport)
			if ipport < 1 or ipport & 0xffff != ipport:
				raise ValueError
	except (KeyError, AttributeError), e:
		ipport = None
	except Exception, e:
		ipport = None
		errors.append('An invalid value for ipport was given: "%s"' % form['ipport'])
	if ipport:
		fencedev.addAttribute('ipport', str(ipport))
	else:
		fencedev.removeAttribute('ipport')

	use_ssl = form.has_key('ssl') and form['ssl'].lower() in ('1', 'on', 'true')
	if use_ssl:
		fencedev.addAttribute('ssl', '1')
	else:
		fencedev.removeAttribute('ssl')

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	return errors

def val_vmware_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)
	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fencedev.addAttribute('secure', '1')
	else:
		fencedev.removeAttribute('secure')

	try:
		vmware_type = form['vmware_type'].strip()
		if not vmware_type in ('esx', 'server1', 'server2'):
			vmware_type = None
			raise Exception, 'blank'
		fencedev.addAttribute('vmware_type', vmware_type)
	except Exception, e:
		try:
			fencedev.removeAttribute('vmware_type')
		except:
			pass

	if not vmware_type in ('server1', 'server2'):
		try:
			vmware_dc = form['vmware_datacenter'].strip()
			if not vmware_dc:
				raise Exception, 'blank'
			fencedev.addAttribute('vmware_datacenter', vmware_dc)
		except Exception, e:
			try:
				fencedev.removeAttribute('vmware_datacenter')
			except:
				pass
	else:
		try:
			fencedev.removeAttribute('vmware_datacenter')
		except:
			pass

	return errors

def val_cisco_ucs_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	try:
		ipport = form['ipport'].strip()
		if ipport:
			ipport = int(ipport)
			if ipport < 1 or ipport & 0xffff != ipport:
				raise ValueError
	except (KeyError, AttributeError), e:
		ipport = None
	except Exception, e:
		ipport = None
		errors.append('An invalid value for ipport was given: "%s"' % form['ipport'])
	if ipport:
		fencedev.addAttribute('ipport', str(ipport))
	else:
		fencedev.removeAttribute('ipport')

	use_ssl = form.has_key('ssl') and form['ssl'].lower() in ('1', 'on', 'true')
	if use_ssl:
		fencedev.addAttribute('ssl', '1')
	else:
		fencedev.removeAttribute('ssl')

	try:
		suborg = form['suborg'].strip()
		if not suborg:
			raise Exception, 'blank'
		fencedev.addAttribute('suborg', suborg)
	except Exception, e:
		fencedev.removeAttribute('suborg')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_rhevm_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	try:
		ipport = form['ipport'].strip()
		if ipport:
			ipport = int(ipport)
			if ipport < 1 or ipport & 0xffff != ipport:
				raise ValueError
	except (KeyError, AttributeError), e:
		ipport = None
	except Exception, e:
		ipport = None
		errors.append('An invalid value for ipport was given: "%s"' % form['ipport'])
	if ipport:
		fencedev.addAttribute('ipport', str(ipport))
	else:
		fencedev.removeAttribute('ipport')

	use_ssl = form.has_key('ssl') and form['ssl'].lower() in ('1', 'on', 'true')
	if use_ssl:
		fencedev.addAttribute('ssl', '1')
	else:
		fencedev.removeAttribute('ssl')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_cisco_mds_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	try:
		udpport = form['udpport'].strip()
		if udpport:
			udpport = int(udpport)
			if udpport < 1 or udpport & 0xffff != udpport:
				raise ValueError
	except (KeyError, AttributeError), e:
		udpport = None
	except Exception, e:
		udpport = None
		errors.append('An invalid value for udpport was given: "%s"' % form['udpport'])
	if udpport:
		fencedev.addAttribute('udpport', str(udpport))
	else:
		fencedev.removeAttribute('udpport')

	try:
		snmp_community = form['community'].strip()
	except Exception, e:
		snmp_community = None
	if snmp_community:
		fencedev.addAttribute('community', snmp_community)
	else:
		fencedev.removeAttribute('community')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	try:
		pwd = form['snmp_priv_passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('snmp_priv_passwd') or form['snmp_priv_passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['snmp_priv_passwd']
		fencedev.addAttribute('snmp_priv_passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('snmp_priv_passwd')
		except:
			pass

	try:
		pwd_script = form['snmp_priv_passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('snmp_priv_passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('snmp_priv_passwd_script')
		except:
			pass

	try:
		snmp_version = form['snmp_version'].strip()
		if not snmp_version in ('1', '2c', '3'):
			snmp_version = None
			raise Exception, 'blank'
		fencedev.addAttribute('snmp_version', snmp_version)
	except Exception, e:
		try:
			fencedev.removeAttribute('snmp_version')
		except:
			pass

	try:
		snmp_sec_level = form['snmp_sec_level'].strip()
		if not snmp_sec_level in ('noAuthNoPriv', 'authNoPriv', 'authPriv'):
			snmp_sec_level = None
			raise Exception, 'blank'
		fencedev.addAttribute('snmp_sec_level', snmp_sec_level)
	except Exception, e:
		try:
			fencedev.removeAttribute('snmp_sec_level')
		except:
			pass

	try:
		snmp_auth_prot = form['snmp_auth_prot'].strip()
		if not snmp_auth_prot in ('MD5', 'SHA'):
			snmp_auth_prot = None
			raise Exception, 'blank'
		fencedev.addAttribute('snmp_auth_prot', snmp_auth_prot)
	except Exception, e:
		try:
			fencedev.removeAttribute('snmp_auth_prot')
		except:
			pass

	try:
		snmp_priv_prot = form['snmp_priv_prot'].strip()
		if not snmp_priv_prot in ('DES', 'AES'):
			snmp_priv_prot = None
			raise Exception, 'blank'
		fencedev.addAttribute('snmp_priv_prot', snmp_priv_prot)
	except Exception, e:
		try:
			fencedev.removeAttribute('snmp_priv_prot')
		except:
			pass

	return errors

def val_noop_fd(dummy, _dummy):
	return []

# non-shared devices

def val_rsa_fd(form, fencedev):
	errors = list()

	try:
		hostname = form['ipaddr'].strip()
		if not hostname:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', hostname)
	except Exception, e:
		errors.append(FD_PROVIDE_HOSTNAME)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_drac_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	try:
		modulename = form['modulename'].strip()
		if not modulename:
			raise Exception, 'blank'
		fencedev.addAttribute('modulename', modulename)
	except Exception, e:
		try:
			fencedev.removeAttribute('modulename')
		except:
			pass

	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fencedev.addAttribute('secure', '1')
		fencedev.addAttribute('agent', 'fence_drac5')
	else:
		fencedev.removeAttribute('secure')
		fencedev.addAttribute('agent', 'fence_drac')

	return errors

def val_rps10_fd(form, fencedev):
	errors = list()

	try:
		device = form['device'].strip()
		if not device:
			raise Exception, 'blank'
		fencedev.addAttribute('device', device)
	except Exception, e:
		errors.append(FD_PROVIDE_DEVICE)

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fencedev.addAttribute('port', port)
	except Exception, e:
		errors.append(FD_PROVIDE_PORT)

	return errors

def val_ipmilan_fd(form, fencedev):
	errors = list()

	try:
		ip = form['ipaddr'].strip()
		if not ip:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', ip)
	except Exception, e:
		errors.append(FD_PROVIDE_IP)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	try:
		auth_type = form['auth'].strip().lower()
		if not auth_type or auth_type == 'none':
			fencedev.removeAttribute('auth')
		elif auth_type == 'password' or auth_type == 'md5':
			fencedev.addAttribute('auth', auth_type)
		else:
			errors.append(FD_PROVIDE_IPMILAN_AUTH)
	except KeyError, e:
		fencedev.removeAttribute('auth')
	except Exception, e:
		errors.append(FD_PROVIDE_IPMILAN_AUTH)

	try:
		privlvl = form['privlvl'].strip().upper()
		if not privlvl in ('CALLBACK', 'USER', 'OPERATOR', 'ADMINISTRATOR'):
			privlvl = None
			raise Exception, 'blank'
		fencedev.addAttribute('privlvl', privlvl)
	except Exception, e:
		try:
			fencedev.removeAttribute('privlvl')
		except:
			pass

	try:
		lanplus = form['lanplus'].strip().lower()
		if not lanplus or lanplus == '0' or lanplus == 'false' or lanplus == 'off':
			fencedev.removeAttribute('lanplus')
		else:
			fencedev.addAttribute('lanplus', '1')
	except Exception, e:
		fencedev.removeAttribute('lanplus')

	return errors

def val_ilo_fd(form, fencedev):
	errors = list()

	try:
		hostname = form['hostname'].strip()
		if not hostname:
			raise Exception, 'blank'
		fencedev.addAttribute('hostname', hostname)
	except Exception, e:
		errors.append(FD_PROVIDE_HOSTNAME)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	return errors

def val_ilo_mp_fd(form, fencedev):
	errors = list()

	try:
		hostname = form['ipaddr'].strip()
		if not hostname:
			raise Exception, 'blank'
		fencedev.addAttribute('ipaddr', hostname)
	except Exception, e:
		errors.append(FD_PROVIDE_HOSTNAME)

	try:
		log = form['login'].strip()
		if not log:
			raise Exception, 'blank'
		fencedev.addAttribute('login', log)
	except Exception, e:
		errors.append(FD_PROVIDE_LOGIN)

	try:
		power_wait = form['power_wait'].strip()
		if power_wait:
			power_wait = int(power_wait)
			if power_wait < 0:
				raise ValueError
	except (KeyError, AttributeError), e:
		power_wait = None
	except Exception, e:
		power_wait = None
		errors.append('An invalid value for power_wait was given: "%s"' % form['power_wait'])
	if power_wait:
		fencedev.addAttribute('power_wait', str(power_wait))
	else:
		fencedev.removeAttribute('power_wait')

	has_passwd = False
	try:
		pwd = form['passwd'].strip()
		if not pwd:
			# Allow passwords that consist of only spaces.
			if not form.has_key('passwd') or form['passwd'] == '':
				raise Exception, 'blank'
			else:
				pwd = form['passwd']
		fencedev.addAttribute('passwd', pwd)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd')
		except:
			pass

	try:
		pwd_script = form['passwd_script'].strip()
		if not pwd_script:
			raise Exception, 'blank'
		fencedev.addAttribute('passwd_script', pwd_script)
		has_passwd = True
	except Exception, e:
		try:
			fencedev.removeAttribute('passwd_script')
		except:
			pass

	if not has_passwd:
		errors.append(FD_PROVIDE_PASSWD)

	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fencedev.addAttribute('secure', '1')
	else:
		fencedev.removeAttribute('secure')

	try:
		ipport = form['ipport'].strip()
		if ipport:
			ipport = int(ipport)
			if ipport < 1 or ipport & 0xffff != ipport:
				raise ValueError
	except (KeyError, AttributeError), e:
		ipport = None
	except Exception, e:
		ipport = None
		errors.append('An invalid value for ipport was given: "%s"' % form['ipport'])
	if ipport:
		fencedev.addAttribute('ipport', str(ipport))
	else:
		fencedev.removeAttribute('ipport')

	try:
		cmd_prompt = form['cmd_prompt'].strip()
	except Exception, e:
		cmd_prompt = None
	if cmd_prompt:
		fencedev.addAttribute('cmd_prompt', cmd_prompt)
	else:
		fencedev.removeAttribute('cmd_prompt')

	return errors

FD_VALIDATE = {
	'fence_apc':			val_apc_fd,
	'fence_wti':			val_wti_fd,
	'fence_brocade':		val_brocade_fd,
	'fence_vixel':			val_vixel_fd,
	'fence_gnbd':			val_gnbd_fd,
	'fence_sanbox2':		val_sanbox2_fd,
	'fence_bladecenter':	val_bladecenter_fd,
	'fence_mcdata':			val_mcdata_fd,
	'fence_egenera':		val_egenera_fd,
	'fence_bullpap':		val_bullpap_fd,
	'fence_lpar':			val_lpar_fd,
	'fence_rhevm':			val_rhevm_fd,
	'fence_cisco_ucs':		val_cisco_ucs_fd,
	'fence_cisco_mds':		val_cisco_mds_fd,
	'fence_ifmib':			val_cisco_mds_fd,
	'fence_vmware':			val_vmware_fd,
	'fence_vmware_soap':	val_vmware_soap_fd,
	'fence_xvm':			val_noop_fd,
	'fence_scsi':			val_noop_fd,
	'fence_ilo':			val_ilo_fd,
	'fence_ilo_mp':			val_ilo_mp_fd,
	'fence_ipmilan':		val_ipmilan_fd,
	'fence_drac':			val_drac_fd,
	'fence_drac5':			val_drac_fd,
	'fence_rsa':			val_rsa_fd,
	'fence_rsb':			val_rsa_fd, # same params as rsa
	'fence_rps10':			val_rps10_fd,
	'fence_manual':			val_noop_fd
}

def validate_fencedevice(form, model, fencedev, fence_edit=False):
	try:
		fence_name = form['name'].strip()
		if not fence_name:
			raise Exception, 'blank'
		fence_name = makeNCName(fence_name)
	except Exception, e:
		return [ FD_PROVIDE_NAME ]

	name_change = False
	if fence_edit is True:
		try:
			old_fence_name = form['orig_name'].strip()
			if not old_fence_name:
				raise Exception, 'blank'
		except Exception, e:
			return [ FD_PROVIDE_NAME ]

		if old_fence_name != fence_name:
			if model.getFenceDeviceByName(fence_name) is not None:
				return [ FD_PROVIDE_NAME ]
			name_change = True
	else:
		if model.getFenceDeviceByName(fence_name) is not None:
			return [ FD_PROVIDE_NAME ]

	try:
		fence_agent = form['fence_type'].strip()
		if not fence_agent:
			raise Exception, 'blank agent'
	except Exception, e:
		return [ FD_PROVIDE_AGENT ]

	fencedev.addAttribute('name', fence_name)
	fencedev.addAttribute('agent', fence_agent)

	try:
		ret = FD_VALIDATE[fence_agent](form, fencedev)
		if len(ret) < 1 and name_change is True:
			try:
				model.rectifyNewFencedevicenameWithFences(old_fence_name, fence_name)
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('vfd0: %s %r %s' \
						% (fence_agent, e, str(e)))
				return [ FD_NEW_FAIL % fence_agent ]
		return ret
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vfd1: %s %r %s' % (fence_agent, e, str(e)))
		return [ FD_NEW_FAIL % fence_agent ]

# Validation Methods for Fence Instances

def val_apc_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	try:
		switch = form['switch'].strip()
		if not switch:
			raise KeyError, 'blank'
		fenceinst.addAttribute('switch', switch)
	except KeyError, e:
		# switch is optional
		fenceinst.removeAttribute('switch')
	except Exception, e:
		errors.append(FI_PROVIDE_SWITCH)

	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fenceinst.addAttribute('secure', '1')
	else:
		fenceinst.removeAttribute('secure')

	return errors

def val_wti_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fenceinst.addAttribute('secure', '1')
	else:
		fenceinst.removeAttribute('secure')

	return errors

def val_brocade_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_vixel_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_gnbd_fi(form, fenceinst):
	errors = list()

	try:
		ip = form['ipaddress'].strip()
		if not ip:
			raise Exception, 'blank'
		fenceinst.addAttribute('ipaddress', ip)
	except Exception, e:
		errors.append(FI_PROVIDE_IPADDRESS)

	return errors

def val_sanbox2_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_bladecenter_fi(form, fenceinst):
	errors = list()

	try:
		blade = form['blade'].strip()
		if not blade:
			raise Exception, 'blank'
		fenceinst.addAttribute('blade', blade)
	except Exception, e:
		errors.append(FI_PROVIDE_BLADE)

	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fenceinst.addAttribute('secure', '1')
	else:
		fenceinst.removeAttribute('secure')

	return errors

def val_mcdata_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_egenera_fi(form, fenceinst):
	errors = list()

	try:
		lpan = form['lpan'].strip()
		if not lpan:
			raise Exception, 'blank'
		fenceinst.addAttribute('lpan', lpan)
	except Exception, e:
		errors.append(FI_PROVIDE_ELPAN)

	try:
		pserver = form['pserver'].strip()
		if not pserver:
			raise Exception, 'blank'
		fenceinst.addAttribute('pserver', pserver)
	except Exception, e:
		errors.append(FI_PROVIDE_ELPAN)

	return errors

def val_bullpap_fi(form, fenceinst):
	errors = list()

	try:
		domain = form['domain'].strip()
		if not domain:
			raise Exception, 'blank'
		fenceinst.addAttribute('domain', domain)
	except Exception, e:
		errors.append(FI_PROVIDE_DOMAIN)

	return errors

def val_xvm_fi(form, fenceinst):
	errors = list()

	try:
		domain = form['domain'].strip()
		if not domain:
			raise Exception, 'blank'
		fenceinst.addAttribute('domain', domain)
	except Exception, e:
		errors.append(FI_PROVIDE_DOMAIN)

	return errors

def val_scsi_fi(form, fenceinst):
	errors = list()

	try:
		nodename = form['node'].strip()
		if not nodename:
			raise Exception, 'blank'
		fenceinst.addAttribute('node', nodename)
	except Exception, e:
		errors.append(FI_PROVIDE_NODENAME)

	return errors

def val_lpar_fi(form, fenceinst):
	errors = list()

	try:
		partition = form['partition'].strip()
		if not partition:
			raise Exception, 'blank'
		fenceinst.addAttribute('partition', partition)
	except Exception, e:
		errors.append(FI_PROVIDE_PARTITION)

	try:
		managed = form['managed'].strip()
		if not managed:
			raise Exception, 'blank'
		fenceinst.addAttribute('managed', managed)
	except Exception, e:
		errors.append(FI_PROVIDE_MANAGED)

	fenceinst.addAttribute('secure', '1')

	return errors

def val_vmware_soap_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		fenceinst.removeAttribute('port')
		port = None

	try:
		uuid = form['uuid'].strip()
		if not uuid:
			raise Exception, 'blank'
		fenceinst.addAttribute('uuid', uuid)
	except Exception, e:
		fenceinst.removeAttribute('uuid')
		uuid = None

	if not port and not uuid:
		errors.append('Either port or UUID must be provided.')

	return errors

def val_vmware_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	use_ssh = form.has_key('secure') and (form['secure'] == '1' or form['secure'].lower() == 'true')
	if use_ssh:
		fenceinst.addAttribute('secure', '1')
	else:
		fenceinst.removeAttribute('secure')

	return errors

def val_cisco_mds_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_cisco_ucs_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_rhevm_fi(form, fenceinst):
	errors = list()

	try:
		port = form['port'].strip()
		if not port:
			raise Exception, 'blank'
		fenceinst.addAttribute('port', port)
	except Exception, e:
		errors.append(FI_PROVIDE_PORT)

	return errors

def val_manual_fi(form, fenceinst):
	nodename = form.get('nodename')
	if nodename:
		fenceinst.addAttribute('nodename', nodename)
	return []

def val_noop_fi(dummy, _dummy):
	return []

FI_VALIDATE = {
	'fence_apc':			val_apc_fi,
	'fence_wti':			val_wti_fi,
	'fence_brocade':		val_brocade_fi,
	'fence_vixel':			val_vixel_fi,
	'fence_gnbd':			val_gnbd_fi,
	'fence_sanbox2':		val_sanbox2_fi,
	'fence_bladecenter':	val_bladecenter_fi,
	'fence_mcdata':			val_mcdata_fi,
	'fence_egenera':		val_egenera_fi,
	'fence_bullpap':		val_bullpap_fi,
	'fence_manual':			val_manual_fi,
	'fence_xvm':			val_xvm_fi,
	'fence_scsi':			val_scsi_fi,
	'fence_lpar':			val_lpar_fi,
	'fence_rhevm':			val_rhevm_fi,
	'fence_cisco_ucs':		val_cisco_ucs_fi,
	'fence_cisco_mds':		val_cisco_mds_fi,
	'fence_ifmib':			val_cisco_mds_fi,
	'fence_vmware':			val_vmware_fi,
	'fence_vmware_soap':	val_vmware_soap_fi,
	'fence_ilo':			val_noop_fi,
	'fence_ilo_mp':			val_noop_fi,
	'fence_ipmilan':		val_noop_fi,
	'fence_drac':			val_noop_fi,
	'fence_drac5':			val_noop_fi,
	'fence_rsa':			val_noop_fi,
	'fence_rsb':			val_noop_fi,
	'fence_rps10':			val_noop_fi
}

def validate_fenceinstance(form, parent_name, fence_agent):
	try:
		if not parent_name.strip():
			return (FD_VAL_FAIL, [ FI_PROVIDE_PARENT ])
	except:
		return (FD_VAL_FAIL, [ FI_PROVIDE_PARENT ])

	fenceinst = Device()
	fenceinst.addAttribute('name', parent_name)
	fenceinst.setAgentType(fence_agent)

	if form.has_key('option'):
		option = form['option'].strip()
		if option:
			fenceinst.addAttribute('option', option)

	try:
		ret = FI_VALIDATE[fence_agent](form, fenceinst)
		if len(ret) > 0:
			return (FD_VAL_FAIL, ret)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('vfi0: %s: %r %s' % (fence_agent, e, str(e)))
		return (FD_VAL_FAIL, [ FI_NEW_FAIL % fence_agent ])

	return (FD_VAL_SUCCESS, fenceinst)
