# Copyright (C) 2006-2011 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from AccessControl import getSecurityManager
from ricci_communicator import RicciCommunicator
from LuciZopePerm import isAdmin
from LuciSyslog import get_logger

from conga_constants import CLUSTER_FOLDER_PATH, BATCH_ID, TASKTYPE, \
	FLAG_DESC, CLUSTER_ADD, LAST_STATUS, \
	STORAGE_FOLDER_PATH, LUCI_DEBUG_MODE

# Cluster node exception attribute flags
CLUSTER_NODE_NEED_AUTH	= 0x01
CLUSTER_NODE_NOT_MEMBER	= 0x02
CLUSTER_NODE_ADDED		= 0x04

luci_log = get_logger()

def getClusterNode(self, nodename, clustername):
	try:
		path = str('%s%s/%s' % (CLUSTER_FOLDER_PATH, clustername, nodename))
		cluster_node = self.restrictedTraverse(path)
		if not cluster_node:
			return None
		return cluster_node
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('getClusterNode0: %s %s: %r %s' \
				% (nodename, clustername, e, str(e)))
	return None

def getStorageNode(self, nodename):
	try:
		path = str('%s%s' % (STORAGE_FOLDER_PATH, nodename))
		storage_node = self.restrictedTraverse(path)
		if not storage_node:
			raise Exception, 'no storage node found'
		return storage_node
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('getStorageNode0: %s: %r %s' \
				% (nodename, e, str(e)))
	return None

def testNodeStatus(node, flag_mask):
	try:
		flags = node.getProperty('flags')
		if flags is None:
			return False
		return flags & flag_mask != 0
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('testNodeStatus0: %r %s' % (e, str(e)))
	return False

def setNodeStatus(node, flag_mask):
	try:
		flags = node.getProperty('flags')
		if flags is None:
			flags = 0
		node.manage_changeProperties({ 'flags': flags | flag_mask })
	except:
		try:
			node.manage_addProperty('flags', flag_mask, 'int')
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('setNodeStatus0: %r %s' \
					% (e, str(e)))

def clearNodeStatus(node, flag_mask):
	try:
		flags = node.getProperty('flags')
		if flags is None:
			return
		if flags & flag_mask != 0:
			node.manage_changeProperties({ 'flags': flags & ~flag_mask })
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('clearNodeStatus0: %r %s' \
				% (e, str(e)))

def del_node_flag(self, cluname, agent, batchid=None):
	path = str('%s%s' % (CLUSTER_FOLDER_PATH, cluname))
	objname = '%s____flag' % agent

	try:
		flag_obj = self.restrictedTraverse(str('%s/%s' % (path, objname)))
		cluster_folder = self.restrictedTraverse(path)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DNF0: %s/%s: %s %r %s' \
				% (path, objname, flag_obj[0], e, str(e)))
		return False

	if batchid:
		try:
			bid = flag_obj.getProperty(BATCH_ID)
			if bid != batchid:
				raise Exception, 'mismatch: got %s, wanted %s' % (bid, batchid)
		except Exception, e1:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('DNF1: unable to find flag with bid: %s for %s/%s: %r %s' % (batchid, path, objname, e1, str(e1)))
			return False

	try:
		cluster_folder.manage_delObjects([objname])
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('DNF2: %s/%s: %r %s' \
				% (path, objname, e, str(e)))
		return False
	return True

def set_node_flag(self, cluname, agent, batchid, task, desc):
	path = str('%s%s' % (CLUSTER_FOLDER_PATH, cluname))
	batch_id = str(batchid)
	objname = '%s____flag' % agent

	objpath = ''
	try:
		cluster_folder = self.restrictedTraverse(path)
		cluster_folder.manage_addProduct['ManagedSystem'].addManagedSystem(objname)
		objpath = str('%s/%s' % (path, objname))
		flag = self.restrictedTraverse(objpath)
		flag.manage_addProperty(BATCH_ID, batch_id, 'string')
		flag.manage_addProperty(TASKTYPE, task, 'string')
		flag.manage_addProperty(FLAG_DESC, desc, 'string')
	except Exception, e:
		errmsg = 'Error creating flag (bid: %s task: %s desc: %s) at %s: %s' \
			% (batch_id, task, desc, objpath, str(e))
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('SNF0: %r %s' % (e, errmsg))
		raise Exception, errmsg

def NodeBusy(self, clustername, nodename, rc=None):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_folder = self.restrictedTraverse(path)
		if not cluster_folder:
			raise Exception, 'cannot find database object at %s' % path
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug('NodeBusy0: %s %r %s' \
				% (clustername, e, str(e)))
		return None

	flagname = '%s____flag' % nodename

	try:
		items = cluster_folder.objectItems('ManagedSystem')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug('NodeBusy1: error getting flags for (%s, %s): %r %s' \
				% (clustername, nodename, e, str(e)))
		return None

	for item in items:
		if item[0] != flagname:
			continue

		# A flag already exists. Check to see whether we're done.
		if rc is None:
			try:
				rc = RicciCommunicator(nodename)
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.info('NodeBusy2: ricci error %s: %r %s' \
						% (nodename, e, str(e)))
				# We can't know if we're done or not; err on the
				# side of caution.
				return True

		if not rc.authed():
			try:
				snode = getStorageNode(self, nodename)
				setNodeStatus(snode, CLUSTER_NODE_NEED_AUTH)
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('NodeBusy3: %s: %r %s' \
						% (nodename, e, str(e)))
			if LUCI_DEBUG_MODE is True:
				luci_log.info('%s not authenticated' % item[0])
			# The comment above applies here, too.
			return True

		batch_id = item[1].getProperty(BATCH_ID)
		batch_ret = rc.batch_status(batch_id)
		finished = batch_ret[0]
		if finished is True or finished == -1:
			if finished == -1:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('NodeBusy5: batch error: %s' \
						% batch_ret[1])

			try:
				cluster_folder.manage_delObjects([flagname])
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.info('NodeBusy6: %s: %r %s' % (flagname, e, str(e)))
			return False

		# Not finished, so don't remove the flag.
		return True

	# If this code is ever reached, no flags exist for the node in question.
	return False

def resolve_nodename(self, clustername, nodename):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_folder = self.restrictedTraverse(path)
		objs = cluster_folder.objectItems('Folder')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('RNN0: error for %s/%s: %r %s' \
				% (nodename, clustername, e, str(e)))
		return nodename

	for obj in objs:
		try:
			if obj[0].find(nodename) != (-1):
				return obj[0]
		except:
			continue

	if LUCI_DEBUG_MODE is True:
		luci_log.debug_verbose('RNN1: failed for %s/%s: nothing found' \
			% (nodename, clustername))
	return nodename

def resolveClusterChanges(self, clustername, model):
	try:
		mb_nodes = model.getNodes()
		if not mb_nodes or not len(mb_nodes):
			raise Exception, 'node list is empty'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('RCC0: no model builder nodes found for %s: %r %s' % (clustername, e, str(e)))
		return 'Unable to find cluster nodes for %s' % clustername

	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_folder = self.restrictedTraverse(path)
		if not cluster_folder:
			raise Exception, 'cluster node is none'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug('RCC1: cant find cluster node for %s: %r %s' \
				% (clustername, e, str(e)))
		return 'Unable to find an entry for %s in the Luci database.' % clustername

	try:
		db_nodes = map(lambda x: x[0], cluster_folder.objectItems('Folder'))
		if not db_nodes or not len(db_nodes):
			raise Exception, 'no database nodes'
	except Exception, e:
		# Should we just create them all? Can this even happen?
		if LUCI_DEBUG_MODE is True:
			luci_log.debug('RCC2: error: %r %s' % (e, str(e)))
		return 'Unable to find database entries for any nodes in %s' % clustername

	same_host = lambda x, y: x == y or x[:len(y) + 1] == y + '.' or y[:len(x) + 1] == x + '.'

	# this is a really great algorithm.
	missing_list = list()
	new_list = list()
	for i in mb_nodes:
		for j in db_nodes:
			f = 0
			if same_host(i, j):
				f = 1
				break
		if not f:
			new_list.append(i)

	for i in db_nodes:
		for j in mb_nodes:
			f = 0
			if same_host(i, j):
				f = 1
				break
		if not f:
			missing_list.append(i)

	messages = list()
	for i in missing_list:
		try:
			# or alternately
			# new_node = cluster_folder.restrictedTraverse(i)
			# #setNodeStatus(self, new_node, CLUSTER_NODE_NOT_MEMBER)
			cluster_folder.delObjects([i])
			messages.append('Node "%s" is no longer in a member of cluster "%s." It has been deleted from the management interface for this cluster.' % (i, clustername))
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VCC3: deleted node %s' % i)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VCC4: delObjects: %s: %r %s' \
					% (i, e, str(e)))

	new_flags = CLUSTER_NODE_NEED_AUTH | CLUSTER_NODE_ADDED
	for i in new_list:
		try:
			cluster_folder.manage_addFolder(i)
			new_node = cluster_folder.restrictedTraverse(str(i))
			setNodeStatus(self, new_node, new_flags)
			messages.append('A new cluster node, "%s," is now a member of cluster "%s." It has been added to the management interface for this cluster, but you must authenticate to it in order for it to be fully functional.' % (i, clustername))
		except Exception, e:
			messages.append('A new cluster node, "%s," is now a member of cluster "%s,". but it has not been added to the management interface for this cluster as a result of an error creating a database entry for it.' % (i, clustername))
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('VCC5: addFolder: %s/%s: %r %s' \
					% (clustername, i, e, str(e)))

	return messages

def buildClusterCreateFlags(self, batch_map, clustername):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_folder = self.restrictedTraverse(path)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('buildCCF0: no cluster folder at %s: %r %s' \
				% (path, e, str(e)))
		return None

	for key in batch_map.keys():
		try:
			key = str(key)
			batch_id = str(batch_map[key])
			# This suffix needed to avoid name collision
			objname = '%s____flag' % key

			cluster_folder.manage_addProduct['ManagedSystem'].addManagedSystem(objname)

			# now designate this new object properly
			objpath = str('%s/%s' % (path, objname))
			flag = self.restrictedTraverse(objpath)

			flag.manage_addProperty(BATCH_ID, batch_id, 'string')
			flag.manage_addProperty(TASKTYPE, CLUSTER_ADD, 'string')
			flag.manage_addProperty(LAST_STATUS, 0, 'int')
			flag.manage_addProperty(FLAG_DESC,
				'Creating node "%s" for cluster "%s"' % (key, clustername),
				'string')
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('buildCCF1: error creating flag for %s: %r %s' % (key, e, str(e)))

def manageCluster(self, clustername, node_list, cluster_os):
	try:
		clusters = self.restrictedTraverse(str(CLUSTER_FOLDER_PATH))
		if not clusters:
			raise Exception, 'cannot find the cluster entry in the DB'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MC0: %s: %r %s' % (clustername, e, str(e)))
		return 'Unable to create cluster %s: the cluster directory is missing.' % clustername

	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		newCluster = self.restrictedTraverse(path)
		if newCluster:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('MC1: cluster %s: already exists' \
					% clustername)
			return 'A cluster named %s is already managed by Luci' % clustername
	except:
		pass

	try:
		clusters.manage_addFolder(clustername)
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		newCluster = self.restrictedTraverse(path)
		if not newCluster:
			raise Exception, 'unable to create the cluster DB entry for %s' % clustername
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MC2: %s: %r %s' % (clustername, e, str(e)))
		return 'Unable to create cluster %s: %s' % (clustername, str(e))

	try:
		newCluster.manage_acquiredPermissions([])
		newCluster.manage_role('View', ['Access contents information', 'View'])
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MC3: %s: %r %s' % (clustername, e, str(e)))
		try:
			clusters.manage_delObjects([clustername])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('MC4: %s: %r %s' \
					% (clustername, e, str(e)))
		return 'Unable to set permissions on new cluster: %s: %s' \
			% (clustername, str(e))

	try:
		newCluster.manage_addProperty('cluster_os', cluster_os, 'string')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MC5: %s: %s: %r %s' \
				% (clustername, cluster_os, e, str(e)))

	for i in node_list:
		host = node_list[i]['host']

		try:
			newCluster.manage_addFolder(host)
			path = str('%s%s/%s' % (CLUSTER_FOLDER_PATH, clustername, host))
			newSystem = self.restrictedTraverse(path)
			if not newSystem:
				raise Exception, 'unable to create cluster system DB entry for node %s' % host
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			try:
				clusters.manage_delObjects([clustername])
			except Exception, e:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('MC6: %s: %s: %r %s' \
						% (clustername, host, e, str(e)))

			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('MC7: %s: %s: %r %s' \
					% (clustername, host, e, str(e)))
			return 'Unable to create cluster node %s for cluster %s: %s' \
				% (host, clustername, str(e))

	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
		if not ssystem:
			raise Exception, 'The storage DB entry is missing'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('MC8: %s: %s: %r %s' \
				% (clustername, host, e, str(e)))
		return 'Error adding storage node %s: %s' % (host, str(e))

	# Only add storage systems if the cluster and cluster node DB
	# objects were added successfully.
	for i in node_list:
		host = node_list[i]['host']

		try:
			# It's already there, as a storage system, no problem.
			path = str('%s%s' % (STORAGE_FOLDER_PATH, host))
			dummy = self.restrictedTraverse(path)
			continue
		except:
			pass

		try:
			ssystem.manage_addFolder(host)
			path = str('%s%s' % (STORAGE_FOLDER_PATH, host))
			newSystem = self.restrictedTraverse(path)
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('MC9: %s: %s: %r %s' \
					% (clustername, host, e, str(e)))

def createClusterSystems(self, clustername, node_list):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		clusterObj = self.restrictedTraverse(path)
		if not clusterObj:
			raise Exception, 'cluster %s DB entry is missing' % clustername
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CCS0: %s: %r %s' \
				% (clustername, e, str(e)))
		return 'No cluster named "%s" is managed by Luci' % clustername

	for x in node_list:
		i = node_list[x]
		host = str(i['host'])

		try:
			clusterObj.manage_addFolder(host)
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('CCS0a: %s: %s: %r %s' \
					% (clustername, host, e, str(e)))

		try:
			path = str('%s%s/%s' % (CLUSTER_FOLDER_PATH, clustername, host))
			newSystem = self.restrictedTraverse(path)
			if not newSystem:
				raise Exception, 'cluster node DB entry for %s disappeared from under us' % host
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('CCS1: %s: %s: %r %s' \
					% (clustername, host, e, str(e)))
			return 'Unable to create cluster node %s for cluster %s: %s' \
				% (host, clustername, str(e))

	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
		if not ssystem:
			raise Exception, 'storage DB entry is missing'
	except Exception, e:
		# This shouldn't fail, but if it does, it's harmless right now
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CCS2: %s: %r %s' \
				% (clustername, host, e, str(e)))
		return None

	# Only add storage systems if the and cluster node DB
	# objects were added successfully.
	for x in node_list:
		i = node_list[x]
		host = str(i['host'])

		try:
			# It's already there, as a storage system, no problem.
			path = str('%s%s' % (STORAGE_FOLDER_PATH, host))
			dummy = self.restrictedTraverse(path)
			continue
		except:
			pass

		try:
			ssystem.manage_addFolder(host)
			path = str('%s%s' % (STORAGE_FOLDER_PATH, host))
			newSystem = self.restrictedTraverse(path)
			newSystem.manage_acquiredPermissions([])
			newSystem.manage_role('View', [ 'Access contents information' , 'View' ])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('CCS3: %s: %r %s' \
					% (clustername, host, e, str(e)))

def delSystem(self, systemName):
	try:
		ssystem = self.restrictedTraverse(STORAGE_FOLDER_PATH)
		if not ssystem:
			raise Exception, 'storage DB entry is missing'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delSystem0: %s: %r %s' \
				% (systemName, e, str(e)))
		return 'Unable to find storage system %s: %s' % (systemName, str(e))

	try:
		rc = RicciCommunicator(systemName, enforce_trust=False)
		if rc is None:
			raise Exception, 'rc is None'
	except Exception, e:
		try:
			ssystem.manage_delObjects([ systemName ])
		except Exception, e1:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('delSystem1: %s: %r %s' \
					% (systemName, e1, str(e1)))
			return 'Unable to delete the storage system %s' % systemName
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delSystem2: %s: %r %s' \
				% (systemName, e, str(e)))
		return

	# Only unauthenticate if the system isn't a member of
	# a managed cluster.
	cluster_info = rc.cluster_info()
	if not cluster_info:
		cluster_name = None
	elif not cluster_info[0]:
		cluster_name = cluster_info[1]
	else:
		cluster_name = cluster_info[0]

	unauth = False
	if not cluster_name:
		# If it's a member of no cluster, unauthenticate
		unauth = True
	else:
		try:
			path = str('%s%s/%s' % (CLUSTER_FOLDER_PATH, cluster_name, systemName))
			dummy = self.restrictedTraverse(path).objectItems()
		except Exception, e:
			# It's not a member of a managed cluster, so unauthenticate.
			unauth = True

	if unauth is True:
		try:
			rc.unauth()
		except:
			pass

	try:
		ssystem.manage_delObjects([ systemName ])
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delSystem3: %s: %r %s' \
				% (systemName, e, str(e)))
		return 'Unable to delete storage system %s: %s' \
			% (systemName, str(e))

def delCluster(self, clustername):
	try:
		clusters = self.restrictedTraverse(CLUSTER_FOLDER_PATH)
		if not clusters:
			raise Exception, 'clusters DB entry is missing'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delCluster0: %r %s' % (e, str(e)))
		return 'Unable to find cluster %s' % clustername

	err = delClusterSystems(self, clustername)
	if err:
		return err

	try:
		clusters.manage_delObjects([ clustername ])
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delCluster1: %s %r %s' \
				% (clustername, e, str(e)))
		return 'Unable to delete cluster %s' % clustername

def delClusterSystem(self, cluster, systemName):
	try:
		path = str('%s%s' % (STORAGE_FOLDER_PATH, systemName))
		dummy = self.restrictedTraverse(path).objectItems()
	except:
		# It's not a storage system, so unauthenticate.
		try:
			rc = RicciCommunicator(systemName, enforce_trust=False)
			rc.unauth()
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('delClusterSystem0: ricci error for %s: %r %s' % (systemName, e, str(e)))

	try:
		cluster.manage_delObjects([ systemName ])
	except Exception, e:
		err_str = 'Error deleting cluster object %s: %s' % (systemName, str(e))
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delClusterSystem1: %r %s' % (e, err_str))
		return err_str

def delClusterSystems(self, clustername):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster = self.restrictedTraverse(path)
		if not cluster:
			raise Exception, 'cluster DB entry is missing'

		try:
			csystems = getClusterSystems(self, clustername)
			if not csystems or len(csystems) < 1:
				return None
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('delCluSystems0: %r %s' % (e, str(e)))
			return None
	except Exception, er:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('delCluSystems1: error for %s: %r %s' \
				% (clustername, er, str(er)))
		return str(er)

	error_list = list()
	for i in csystems:
		cur_sysname = str(i[0])
		err = delClusterSystem(self, cluster, cur_sysname)
		if err:
			error_list.append('Unable to delete the cluster system %s: %s\n' \
				% (cur_sysname, err))
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('delCluSystems2: %s' % err)
	return ''.join(error_list)

def getSystems(self):
	storage = getStorage(self)
	clusters = getClusters(self)
	storageList = list()
	ret = [{}, [], {}]

	need_auth_hash = {}
	for i in storage:
		storageList.append(i[0])
		if testNodeStatus(i[1], CLUSTER_NODE_NEED_AUTH) is not False:
			need_auth_hash[i[0]] = i[1]

	chash = {}
	for i in clusters:
		csystems = getClusterSystems(self, i[0])
		cslist = list()
		for c in csystems:
			if testNodeStatus(c[1], CLUSTER_NODE_NEED_AUTH) is not False:
				need_auth_hash[c[0]] = c[1]
			cslist.append(c[0])
		chash[i[0]] = cslist

	ret[0] = chash
	ret[1] = storageList
	ret[2] = need_auth_hash
	return ret

def getCluster(self, clustername):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_obj = self.restrictedTraverse(path)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCobj0: %s: %r %s' \
				% (clustername, e, str(e)))
		return None

	if cluster_permission_check(self, cluster_obj):
		return cluster_obj
	return None

def getClusterSystems(self, clustername):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_nodes = self.restrictedTraverse(path).objectItems('Folder')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCSy0: %s: %r %s' \
				% (clustername, e, str(e)))
		return None

	if isAdmin(self):
		return cluster_nodes
	cluster_obj = getCluster(self, clustername)
	if cluster_permission_check(self, cluster_obj):
		return cluster_nodes
	return None

def getClusters(self):
	try:
		clusters = self.restrictedTraverse(CLUSTER_FOLDER_PATH).objectItems('Folder')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GC0: %r %s' % (e, str(e)))
		return None

	if isAdmin(self):
		return clusters
	return check_clusters(self, clusters)

def getStorage(self):
	try:
		storage = self.restrictedTraverse(STORAGE_FOLDER_PATH).objectItems('Folder')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GS0: %r %s' % (e, str(e)))
		return None

	if isAdmin(self):
		return storage
	return allowed_systems(storage)

def check_clusters(self, clusters):
	ret = []
	try:
		user = getSecurityManager().getUser()
		ret = filter(lambda x: user.has_permission('View', x[1]), clusters)
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CC0: %r %s' % (e, str(e)))
	return ret

def cluster_permission_check(self, cluster):
	if isAdmin(self):
		return True

	try:
		user = getSecurityManager().getUser()
		if user.has_permission('View', cluster):
			return True
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('CPC0: %s: %r %s' % (cluster, e, str(e)))
	return False

def allowed_systems(self, systems):
	user = getSecurityManager().getUser()
	return filter(lambda x: user.has_permission('View', x[1]), systems)

def access_to_host_allowed(self, hostname, allowed_systems_list):
	allowed = dict(map(lambda x: [ x[0], None ], allowed_systems_list))
	return allowed.has_key(hostname)

def getRicciAgent(self, clustername, exclude_names=None, exclude_busy=False):
	cluster_folder = None
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		cluster_folder = self.restrictedTraverse(path)
		if not cluster_folder:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug('GRA0: cluster folder %s for %s is missing' \
					% (path, clustername))
			return None

		try:
			perm = cluster_permission_check(self, cluster_folder)
			if not perm:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA1: no permission for %s' \
						% clustername)
				return None
		except Exception, e1:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GRA2: %s: %r %s' \
					% (clustername, e1, str(e1)))
			return None

		nodes = cluster_folder.objectItems('Folder')
		if len(nodes) < 1:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug('GRA2: no cluster nodes for %s found' \
					% clustername)
			raise Exception, 'No cluster nodes were found at %s' % path
	except Exception, e:
		try:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug('GRA3: cluster folder %s for %s is missing: %r %s' % (path, clustername, e, str(e)))

			if cluster_folder is not None and len(cluster_folder.objectItems()) < 1:
				clusters = self.restrictedTraverse(CLUSTER_FOLDER_PATH)
				clusters.manage_delObjects([clustername])
		except Exception, ein:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GRA4: %r %s' % (ein, str(ein)))
		return None

	cluname = clustername.lower()
	for node in nodes:
		hostname = node[0]

		if hostname[-6:] == '__flag':
			continue

		if exclude_names is not None and hostname in exclude_names:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GRA5: %s is in the excluded names list, excluding it' % hostname)
			continue

		try:
			rc = RicciCommunicator(hostname)
			if not rc:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA6: rc is None')
				continue

			ricci_hostname = rc.hostname()
			if not ricci_hostname:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA7: ricci_hostname is blank')
				continue

			clu_info = rc.cluster_info()

			cur_name = str(clu_info[0]).strip().lower()
			if not cur_name:
				cur_name = None
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA8: cluster name is none for %s' \
						% ricci_hostname)

			cur_alias = str(clu_info[1]).strip().lower()
			if not cur_alias:
				cur_alias = None
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA9: no cluster alias for %s' \
						% ricci_hostname)

			if (cur_name is not None and cluname != cur_name) and (cur_alias is not None and cluname != cur_alias):
				if LUCI_DEBUG_MODE is True:
					luci_log.debug('GRA10: node %s reports it\'s in cluster [%s:%s] we expect %s' % (ricci_hostname, clu_info[0], clu_info[1], cluname))
				setNodeStatus(self, node, CLUSTER_NODE_NOT_MEMBER)
				continue

			if not rc.authed():
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA11: %s is not authenticated' \
						% ricci_hostname)
				setNodeStatus(node[1], CLUSTER_NODE_NEED_AUTH)
				continue
		except Exception, eout:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GRA12: %r %s' % (eout, str(eout)))
			continue

		if exclude_busy is True:
			if NodeBusy(self, clustername, ricci_hostname, rc) is not False:
				if LUCI_DEBUG_MODE is True:
					luci_log.debug_verbose('GRA13: %s is busy, excluding' \
						% ricci_hostname)
				continue
		return rc

	if LUCI_DEBUG_MODE is True:
		luci_log.debug('GRA14: no ricci agent could be found for cluster %s' \
			% clustername)
	return None

def getClusterDBObj(self, clustername):
	try:
		cluster_path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		nodelist = self.restrictedTraverse(cluster_path)
		if not nodelist:
			raise Exception, 'no nodelist'
		return nodelist
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCDB0: %s -> %s: %r %s' \
				% (clustername, cluster_path, e, str(e)))
	return None

def getClusterDBNodes(self, clustername):
	try:
		cluster_path = str('%s%s' % (CLUSTER_FOLDER_PATH, clustername))
		nodelist = self.restrictedTraverse(cluster_path).objectItems('Folder')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCDBN0: %s -> %s: %r %s' \
				% (clustername, cluster_path, e, str(e)))
		return []
	return nodelist

def getClusterDBVersion(cluster_folder):
	try:
		cluster_os = str(cluster_folder.getProperty('cluster_os'))
		if not cluster_os:
			raise Exception, 'cluster os is blank'
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCDBV0: %s: %r %s' \
				% (cluster_folder.getId(), e, str(e)))
		cluster_os = None
	return cluster_os

def setClusterDBVersion(cluster_folder, version_str):
	if cluster_folder.getProperty('cluster_os') is None:
		try:
			cluster_folder.manage_addProperty('cluster_os',
				version_str, 'string')
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('SCDBV0: %s: %r %s' \
					% (cluster_folder.getId(), e, str(e)))
			return None
	else:
		try:
			cluster_folder.manage_changeProperties({'cluster_os': version_str })
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('SCDBV1: %s: %r %s' \
					% (cluster_folder.getId(), e, str(e)))
			return None
	return True

def getClusterFlags(self, cluname):
	try:
		path = str('%s%s' % (CLUSTER_FOLDER_PATH, cluname))
		cluster_folder = self.restrictedTraverse(path)
		if not cluster_folder:
			raise Exception, 'cluster_folder is None'
		return cluster_folder.objectItems('ManagedSystem')
	except Exception, e:
		if LUCI_DEBUG_MODE is True:
			luci_log.debug_verbose('GCF0: cluster %s [%s] folder missing: %r %s -- returning empty map' % (cluname, path, e, str(e)))
	return None

def getClusterStatusDB(self, clustername):
	results = list()
	vals = {}

	vals['type'] = 'cluster'
	vals['alias'] = clustername
	vals['name'] = clustername
	vals['error'] = True
	vals['quorate'] = '[unknown]'
	vals['votes'] = '[unknown]'
	vals['minQuorum'] = '[unknown]'
	results.append(vals)

	nodelist = getClusterDBNodes(self, clustername)
	if len(nodelist) < 1:
		luci_log.info('Removing cluster %s because it has no nodes' \
			% clustername)
		try:
			clusters_dir = self.restrictedTraverse(CLUSTER_FOLDER_PATH)
			clusters_dir.manage_delObjects([clustername])
		except Exception, e:
			if LUCI_DEBUG_MODE is True:
				luci_log.debug_verbose('GCSDB0b: %s: %r %s' \
					% (clustername, e, str(e)))
		return results

	for node in nodelist:
		node_val = {}
		node_val['type'] = 'node'
		node_val['name'] = node[0]
		node_val['clustered'] = '[unknown]'
		node_val['online'] = '[unknown]'
		node_val['error'] = True
		results.append(node_val)
	return results
